 /******************************************************************************
  * @file    PT32x030x_ifmc.h
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2022/12/29
  * @brief    This file contains all the functions prototypes for the IFMC firmware library.
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef PT32x030x_IFMC_H
#define PT32x030x_IFMC_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "PT32x030x.h"


/** @addtogroup IFMC
  * @{
  */

/* Exported types ------------------------------------------------------------*/

/**  @brief  IFMC Status
  * @{
  */
typedef enum
{	
	IFMC_WriteEnd=1,
	IFMC_PageEraseEnd,
	IFMC_ChipEraseEnd,
	IFMC_Busy,
	IFMC_CommandError,
	IFMC_KeyError,
	IFMC_AddrError,
	IFMC_Complete,
	IFMC_TimeOut
}IFMC_Status;
/**
  * @}
  */

/* Exported constants --------------------------------------------------------*/
#define IFMC_PAGE_SIZE    	 ((u16)0x200)	 /*!< 512B */
/** @defgroup IFMC_Latency 
  * @{
  */
#define IFMC_Latency_0                (IFMC_CR_WAIT&0x00) 		  /*!< IFMC Zero Latency cycle */
#define IFMC_Latency_1                (IFMC_CR_WAIT&0x20)       /*!< IFMC One Latency cycle */
#define IFMC_Latency_2                (IFMC_CR_WAIT&0x40)       /*!< IFMC Two Latency cycle */
#define IS_IFMC_Latency(SEL)	(((SEL) == IFMC_Latency_0) || \
															 ((SEL) == IFMC_Latency_1) || \
															 ((SEL) == IFMC_Latency_2))
/**
  * @}
  */

/** @defgroup IS_FLASH_PSC_Value
  * @{
  */
#define IS_IFMC_Prescaler(SEL) 	 	 			((SEL)< 64)
/**
  * @}
  */

/** @defgroup IFMC_flash_size_selction 
  * @{
  */
/*-------------------------------------------------------*/ 
/* 		          User Please Select Flash Size        		 */ 
/*-------------------------------------------------------*/ 
#define FLASH_SIZE_32K	(0)				 
#define FLASH_SIZE_64K	(1)	
/*---------------------------------------*/ 
#if FLASH_SIZE_32K
	#define MAINCODE_END_ADDR    ((u32)0x00007FFF) 
#elif FLASH_SIZE_64K
	#define MAINCODE_END_ADDR    ((u32)0x0000FFFF) 
#else
	#define MAINCODE_END_ADDR    ((u32)0x00007FFF) 
#endif
/**
  * @}
  */

/** @brief  IFMC EncryptedArea addr **/ 
#define IS_IFMC_PageAddress(SEL)   ((SEL) < MAINCODE_END_ADDR)
/**
  * @}
  */

/** @defgroup IFMC_flag_selection 
  * @{
  */
#define IFMC_FLAG_WC		IFMC_SR_WC			/*!< Write operation complete */
#define IFMC_FLAG_PEC		IFMC_SR_PEC   	/*!< Page erase operation complete */
#define IFMC_FLAG_CEC		IFMC_SR_CEC  		/*!< Chip erase operation complete */
#define IFMC_FLAG_BSY		IFMC_SR_BSY			/*!< Operation busy */ 
#define IFMC_FLAG_CERR	IFMC_SR_CERR		/*!< Operation command error */
#define IFMC_FLAG_KERR	IFMC_SR_KERR		/*!< Key error */
#define IFMC_FLAG_AERR	IFMC_SR_AERR		/*!< Address error */
#define IFMC_FLAG_ALL		(IFMC_FLAG_WC|IFMC_FLAG_PEC|IFMC_FLAG_CEC|IFMC_FLAG_BSY|IFMC_FLAG_CERR|IFMC_FLAG_KERR|IFMC_FLAG_AERR)
#define IS_IFMC_ClearFLAG(SEL) ((((SEL) & (u16)0xff00) == 0) && ((SEL) != (u16)0x00))
#define IS_IFMC_GETFLAG(SEL)	 (((SEL) == IFMC_FLAG_WC)  		 || \
																((SEL) == IFMC_FLAG_PEC)  	 || \
																((SEL) == IFMC_FLAG_CEC)  	 || \
																((SEL) == IFMC_FLAG_BSY)  	 || \
																((SEL) == IFMC_FLAG_CERR)  	 || \
																((SEL) == IFMC_FLAG_KERR)		 || \
																((SEL) == IFMC_FLAG_AERR))
								
/**
  * @}
  */

/** @defgroup IFMC_interrupt_selection 
  * @{
  */
#define IFMC_IT_WCIE		IFMC_IE_WCIE			/*!< Write operation complete interrupt enable */
#define IFMC_IT_PECIE		IFMC_IE_PECIE   	/*!< Page erase operation complete interrupt enable */
#define IFMC_IT_CECIE		IFMC_IE_CECIE  		/*!< Chip erase operation complete interrupt enable */
#define IFMC_IT_CERIE		IFMC_IE_CERIE			/*!< Operation command error interrupt enable */
#define IFMC_IT_KERIE		IFMC_IE_KERIE			/*!< Key error interrupt enable */
#define IFMC_IT_AERIE		IFMC_IE_AERIE			/*!< Address error interrupt enable */
#define IS_IFMC_IT(SEL)					((SEL<= 0x7F) && (SEL!=0x08))
/**
  * @}
  */

/** @defgroup IFMC_Timeout_definition  **/ 
#define IFMC_ER_PRG_TIMEOUT      ((u32)0x000B0000)
/**
  * @}
  */

/** @brief  IFMC Operation Commands **/ 
#define IFMC_Programming		 (IFMC_CR_CMD&0x00)
#define IFMC_PageErase			 (IFMC_CR_CMD&0x02)
#define IFMC_MainCodeErase	 (IFMC_CR_CMD&0x04)
#define IFMC_WriteOperation		IFMC_CR_MODE
#define IFMC_OperationStarts	IFMC_CR_START
/**
  * @}
  */

/** @defgroup IFMC_Code_Key 
  * @{
  */
#define IFMC_MainCodeKey						((u32)0xADEB0000)   
#define IFMC_ConfigurationCodeKey  	((u32)0xC5AE0000)
/**
  * @}
  */

/* Exported macro ------------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */
void IFMC_SetLatency(u32 IFMC_Latency);
void IFMC_SetPrescaler(u8 Prescaler);
IFMC_Status IFMC_GetIFMCStatus(void);
IFMC_Status IFMC_WaitForLastOperation(u32 Timeout);

IFMC_Status IFMC_ErasePage(u32 Page_Address);
IFMC_Status IFMC_EraseAllPages(void);
IFMC_Status IFMC_ProgramWord(u32 Address, u32 Data);

u32 IFMC_ReadWord(u32 Address);
u16 IFMC_ReadHalfWord(u32 Address);
u8 IFMC_ReadByte(u32 Address);
void IFMC_ITConfig(u32 IFMC_IT, FunctionalState NewState);
FlagStatus IFMC_GetFlagStatus(u32 IFMC_FLAG);
void IFMC_ClearFlag(u32 IFMC_FLAG);
ProtectStatus IFMC_GetReadProtectStatus(void);

/**
  * @}
  */

  
#ifdef __cplusplus
}
#endif

#endif 
