/********************************************************************************
  * @file    pt32x030x_adc.h
  * @author  Application development team
  * @version V1.0.0
  * @date    2022/12/29 
  * @brief   This file contains all the functions prototypes for the ADC firmware 
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

#include "PT32x030x_adc.h"

/** @defgroup ADC
  * @brief ADC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_CR1_MASK           ((u32)0xDAC0F700)   //!< CR1 MASK 
#define Init_CR2_MASK           ((u32)0xFF80FFFF)   //!< CR2 MASK 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/



/**
  * @brief ADC Initialization
  * @param ADCx:ADC Peripheral
  * @param ADC_InitStruct:ADC_InitTypeDef type structure
  * @retval None
  */
void ADC_Init(ADC_TypeDef* ADCx, ADC_InitTypeDef* ADC_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_Prescaler(ADC_InitStruct->ADC_Prescaler));
	assert_param(IS_ADC_Mode(ADC_InitStruct->ADC_Mode));
	assert_param(IS_ADC_TriggerSource(ADC_InitStruct->ADC_TriggerSource));
	assert_param(IS_ADC_TimerTriggerSource(ADC_InitStruct->ADC_TimerTriggerSource));
	assert_param(IS_ADC_ExternalPinTriggerSource(ADC_InitStruct->ADC_ExternalPinTriggerSource));
	assert_param(IS_ADC_Align(ADC_InitStruct->ADC_Align));
	assert_param(IS_ADC_Channel(ADC_InitStruct->ADC_Channel));
	assert_param(IS_ADC_BGVoltage(ADC_InitStruct->ADC_BGVoltage));
	assert_param(IS_ADC_VoltageReferencePositive(ADC_InitStruct->ADC_VoltageReferencePositive));

	tmpreg = ADCx->CR1;
	tmpreg &= Init_CR1_MASK;
	tmpreg |= ((ADC_InitStruct->ADC_Mode)| \
						 (ADC_InitStruct->ADC_TriggerSource)|\
						 (ADC_InitStruct->ADC_TimerTriggerSource)|\
						 (ADC_InitStruct->ADC_ExternalPinTriggerSource)|\
						 (ADC_InitStruct->ADC_Align)|\
						 (ADC_InitStruct->ADC_Channel)|\
						 (ADC_InitStruct->ADC_BGVoltage)|\
						 (ADC_InitStruct->ADC_VoltageReferencePositive));
	ADCx->CR1 = tmpreg;
	
	tmpreg = ADCx->CR2;
	tmpreg &= Init_CR2_MASK;
	tmpreg |= ((ADC_InitStruct->ADC_Prescaler)<<16);
	ADCx->CR2 = tmpreg;
}

/**
  * @brief ADC Structure Initialization
  * @param ADC_InitStruct:ADC_InitTypeDef type structure
  * @retval None
  */
void ADC_StructInit(ADC_InitTypeDef* ADC_InitStruct)
{
	ADC_InitStruct->ADC_Prescaler = 2;
	ADC_InitStruct->ADC_Mode = ADC_Mode_Single;
	ADC_InitStruct->ADC_TriggerSource = ADC_TriggerSource_Software;
	ADC_InitStruct->ADC_TimerTriggerSource = ADC_TimerTriggerSource_TIM1ADC;
	ADC_InitStruct->ADC_ExternalPinTriggerSource = ADC_ExternalPinTriggerSource_ADCETR;
	ADC_InitStruct->ADC_Align = ADC_Align_Right;
	ADC_InitStruct->ADC_Channel = ADC_Channel_0;
	ADC_InitStruct->ADC_BGVoltage = ADC_BGVoltage_BG1v2;
	ADC_InitStruct->ADC_VoltageReferencePositive = ADC_VoltageReferencePositive_VDDA;
}

/**
  * @brief ADC enable command
  * @param ADCx:ADC Peripheral
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void ADC_Cmd(ADC_TypeDef* ADCx, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState != DISABLE)
    {
        /* Set the ADEN bit to Enable the ADC peripheral */
        ADCx->CR1 |= ADC_CR1_EN;
    }
    else
    {
        ADCx->CR1 &= (~ADC_CR1_EN);
    }
}

/**
  * @brief ADC Channel Config
  * @param ADCx:ADC Peripheral
  * @param ADC_Channel:the ADC channel to configure.
	 * @param ADC_Channel:the ADC channel to configure.
	*          This parameter can be one of the following values:
  *            @arg ADC_Channel_0: ADC Channel0 selected
  *            @arg ADC_Channel_1: ADC Channel1 selected
  *            @arg ADC_Channel_2: ADC Channel2 selected
  *            @arg ADC_Channel_3: ADC Channel3 selected
  *            @arg ADC_Channel_4: ADC Channel4 selected
  *            @arg ADC_Channel_5: ADC Channel5 selected
  *            @arg ADC_Channel_6: ADC Channel6 selected
  *            @arg ADC_Channel_7: ADC Channel7 selected
  *            @arg ADC_Channel_8: ADC Channel8 selected
  *            @arg ADC_Channel_9: ADC Channel9 selected
  *            @arg ADC_Channel_10: ADC Channel10 selected
  *            @arg ADC_Channel_11: ADC Channel11 selected
  *            @arg ADC_Channel_12: ADC Channel12 selected
  *            @arg ADC_Channel_13: ADC Channel13 selected
  *            @arg ADC_Channel_14: ADC Channel14 selected
  *            @arg ADC_Channel_15: ADC Channel15 selected
	*            @arg ADC_Channel_16: ADC Channel16 selected
  *            @arg ADC_Channel_17: ADC Channel17 selected
  *            @arg ADC_Channel_18: ADC Channel18 selected
  * @retval None
  */
void ADC_ChannelConfig(ADC_TypeDef* ADCx, u32 ADC_Channel)
{

  /* Check the parameters */
  assert_param(IS_ADC_ALL_PERIPH(ADCx));
  assert_param(IS_ADC_Channel(ADC_Channel));
  /* Clear the ADC Channel Selection bits */
  ADCx->CR1 &=(u32)~ADC_CR1_CHS;
  /* Configure the ADC Channel */
  ADCx->CR1 |= (u32)ADC_Channel;
}

/**
  * @brief ADC Start Of Conversion
  * @param ADCx:ADC Peripheral
  * @retval None
  */
void ADC_StartOfConversion(ADC_TypeDef* ADCx)
{
    /* Check the parameters */
    assert_param(IS_ADC_ALL_PERIPH(ADCx));

    ADCx->CR1 |= ADC_CR1_SOC;
}

/**
  * @brief ADC Get Conversion Value
  * @param ADCx:ADC Peripheral
  * @retval (u16)ADCx->DR
  */
u16 ADC_GetConversionValue(ADC_TypeDef* ADCx)
{
    /* Check the parameters */
    assert_param(IS_ADC_ALL_PERIPH(ADCx));

    /* Return the selected ADC conversion value */
    return (u16) ADCx->DR;
}

/**
  * @brief ADC interrupt Config
  * @param ADCx:ADC Peripheral
  * @param ADC_IT: ADC interrupt source
	* @arg ADC_IT_EOC: ADC end of conversion interrupt                                       
	* @arg ADC_IT_EOS: ADC end of scan interrupt   
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void ADC_ITConfig(ADC_TypeDef* ADCx, u32 ADC_IT, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_ADC_IT(ADC_IT)); 

    if (NewState != DISABLE)
    {
        /* Enable the selected ADC interrupts */
        ADCx->CR1 |= ADC_IT;
    }
    else
    {
        /* Disable the selected ADC interrupts */
        ADCx->CR1 &= (~ADC_IT);
    }
}

/**
  * @brief ADC Get Flag Status
  * @param ADCx:ADC Peripheral
  * @param ADC_FLAG: specifies the flag to check.
	* @arg ADC_FLAG_RDY: ADC ready flag                                       
	* @arg ADC_FLAG_EOC: ADC end of conversion flag
	* @arg ADC_FLAG_EOS: ADC end of scan flag
  * @retval FlagStatus
  */
FlagStatus ADC_GetFlagStatus(ADC_TypeDef* ADCx, u32 ADC_FLAG)
{
    FlagStatus bitstatus = RESET;

    /* Check the parameters */
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_ADC_FLAG(ADC_FLAG));
    /* Check the status of the specified ADC flag */
    if ((ADCx->SR & ADC_FLAG) != (u32)RESET)
    {
        /* ADC_FLAG is set */
        bitstatus = SET;
    }
    else
    {
        /* ADC_FLAG is reset */
        bitstatus = RESET;
    }
    /* Return the ADC_FLAG status */
    return  bitstatus;
}

/**
  * @brief ADC Average Command
  * @param ADCx:ADC Peripheral 
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void ADC_AverageCmd(ADC_TypeDef* ADCx,FunctionalState NewState)
{
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    if(NewState!=DISABLE)
    { 
        ADCx->CR1 |= ADC_CR1_AVGE;
    }
    else
    {
        ADCx->CR1 &= (~ADC_CR1_AVGE);
    }		
}

/**
  * @brief ADC Average Times Config
  * @param ADCx:ADC Peripheral
  * @param Times: the time of average conversion
	* @arg ADC_AverageTimes_1: Average conversion 1 times                                         
	* @arg ADC_AverageTimes_2: Average conversion 2 times                                                                                 
	* @arg ADC_AverageTimes_4: Average conversion 4 times                                                                                  
	* @arg ADC_AverageTimes_8: Average conversion 8 times                                                                                  
	* @arg ADC_AverageTimes_16: Average conversion 16 times                                                                                  
	* @arg ADC_AverageTimes_32: Average conversion 32 times                                                                                  
	* @arg ADC_AverageTimes_64: Average conversion 64 times                                         
	* @arg ADC_AverageTimes_128: Average conversion 128 times                                          
  * @retval None
  */
void ADC_AverageTimesConfig(ADC_TypeDef* ADCx,u32 Times)
{
    u32 tmpreg=0;	
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_ADC_AverageTimes(Times));

    tmpreg = ADCx->CR2;
    tmpreg &= (~ADC_CR2_AVGT);
    tmpreg |= ((Times)<<24);
    ADCx->CR2 = tmpreg;	
}

/**
  * @brief ADC Ready Time Config
  * @param ADCx:ADC Peripheral 
  * @param ReadyTime:0x0-0xFF
  * @retval None
  */
void ADC_ReadyTimeConfig(ADC_TypeDef* ADCx,u32 ReadyTime)
{
    u32 tmpreg=0;
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_ADC_ReadyTime(ReadyTime));

    tmpreg = ADCx->CR2;
    tmpreg &= (~ADC_CR2_RDTC);
    tmpreg |= ReadyTime;
    ADCx->CR2 = tmpreg;	
}

/**
  * @brief ADC Sample Time Config
  * @param ADCx:ADC Peripheral 
  * @param SampleTime:0x0-0x7F
  * @retval None
  */
void ADC_SampleTimeConfig(ADC_TypeDef* ADCx,u32 SampleTime)
{
	u32 tmpreg=0;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_SampleTime(SampleTime));	

	tmpreg = ADCx->CR2;
	tmpreg &= (~ADC_CR2_SMP);
	tmpreg |= ((SampleTime)<<8);
    ADCx->CR2 = tmpreg;	
}

/**
  * @brief ADC calibration Command
  * @param ADCx:ADC Peripheral 
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void ADC_TrimCmd(ADC_TypeDef* ADCx, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState != DISABLE)
    {
        /* Set the CORRE bit to Enable the ADC correction */
        ADCx->CR1 |= ADC_CR1_TME;
    }
    else
    {
        ADCx->CR1 &= (~ADC_CR1_TME);
    }
}

/**
  * @brief ADC calibration value config
  * @param ADCx:ADC Peripheral 
  * @param Value:0x00-0x3F
  * @retval None
  */
void ADC_TrimValueConfig(ADC_TypeDef* ADCx,s32 Value)
{
    u32 tmpreg=0;	
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_ADC_TrimValue(Value));

    tmpreg = ADCx->CCR;
    tmpreg &= (~ADC_CCR_TRIM);
    tmpreg |= (ADC_CCR_KEY|Value);
    ADCx->CCR = tmpreg;			
}

/**
  * @brief ADC Scan Command
  * @param ADCx:ADC Peripheral 
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void ADC_ScanCmd(ADC_TypeDef* ADCx, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState != DISABLE)
    {
        /* Set the SCANE bit to Enable the ADC scan */
        ADCx->CR1 |= ADC_CR1_SCANE;
    }
    else
    {
        ADCx->CR1 &= (~ADC_CR1_SCANE);
    }
}

/**
  * @brief ADC Scan Channel Config
  * @param ADCx:ADC Peripheral 
  * @param ADC_Channel:ADC Peripheral
  * @param ADC_Channel:the ADC channel to configure.
	*          This parameter can be one of the following values:
  *            @arg ADC_Channel_0: ADC Channel0 selected
  *            @arg ADC_Channel_1: ADC Channel1 selected
  *            @arg ADC_Channel_2: ADC Channel2 selected
  *            @arg ADC_Channel_3: ADC Channel3 selected
  *            @arg ADC_Channel_4: ADC Channel4 selected
  *            @arg ADC_Channel_5: ADC Channel5 selected
  *            @arg ADC_Channel_6: ADC Channel6 selected
  *            @arg ADC_Channel_7: ADC Channel7 selected
  *            @arg ADC_Channel_8: ADC Channel8 selected
  *            @arg ADC_Channel_9: ADC Channel9 selected
  *            @arg ADC_Channel_10: ADC Channel10 selected
  *            @arg ADC_Channel_11: ADC Channel11 selected
  *            @arg ADC_Channel_12: ADC Channel12 selected
  *            @arg ADC_Channel_13: ADC Channel13 selected
  *            @arg ADC_Channel_14: ADC Channel14 selected
  *            @arg ADC_Channel_15: ADC Channel15 selected
	*            @arg ADC_Channel_16: ADC Channel16 selected
  *            @arg ADC_Channel_17: ADC Channel17 selected
  *            @arg ADC_Channel_18: ADC Channel18 selected
  * @param ScanChannel: ADC scan channel (0~19)
  * @retval None
  */
void ADC_ScanChannelConfig(ADC_TypeDef* ADCx,u32 ADC_Channel,u32 ScanChannel)
{
    u32 tmpreg=0;	  
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_ADC_Channel(ADC_Channel));
    assert_param(IS_ADC_ScanChannel(ScanChannel));	

    if(ScanChannel <= 5)
    {
        tmpreg = ADCx->SCHR1;	  
        tmpreg &= (~(ADC_SCHR1_CH0<<(ScanChannel*5)));
        tmpreg |= ((ADC_Channel>>16)<<ScanChannel*5);
        ADCx->SCHR1 = tmpreg;
    }
    if((ScanChannel >= 6)&&(ScanChannel <= 11))
    {
        tmpreg = ADCx->SCHR2;	  
        tmpreg &= (~(ADC_SCHR2_CH6<<((ScanChannel-6)*5)));
        tmpreg |= ((ADC_Channel>>16)<<(ScanChannel-6)*5);
        ADCx->SCHR2 = tmpreg;
    }
    if((ScanChannel >= 12)&&(ScanChannel <= 17))
    {
        tmpreg = ADCx->SCHR3;	  
        tmpreg &= (~(ADC_SCHR3_CH12<<((ScanChannel-12)*5)));
        tmpreg |= ((ADC_Channel>>16)<<(ScanChannel-12)*5);
        ADCx->SCHR3 = tmpreg;
    }
     if((ScanChannel >= 18)&&(ScanChannel <= 19))
    {
        tmpreg = ADCx->SCHR4;	  
        tmpreg &= (~(ADC_SCHR4_CH18<<((ScanChannel-18)*5)));
        tmpreg |= ((ADC_Channel>>16)<<(ScanChannel-18)*5);
        ADCx->SCHR4 = tmpreg;
    }
}

/**
  * @brief ADC Scan Channel Number Config
  * @param ADCx:ADC Peripheral 
  * @param ScanNumber: the number of ADC scan channel (2~20)
  * @retval None
  */
void ADC_ScanChannelNumberConfig(ADC_TypeDef* ADCx,u32 ScanNumber)
{
    u32 tmpreg=0;	  
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_ADC_ScanChannelNumber(ScanNumber));	

    tmpreg = ADCx->SCHR4;	  
    tmpreg &= (~ADC_SCHR4_CNT);
    tmpreg |= ((ScanNumber-1) << 16);
    ADCx->SCHR4 = tmpreg;
}

/**
  * @brief ADC DMA Command
  * @param ADCx:ADC Peripheral 
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void ADC_DMACmd(ADC_TypeDef* ADCx, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_ADC_ALL_PERIPH(ADCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState != DISABLE)
    {
        /* Set the DMAE bit to Enable the ADC DMA transition*/
        ADCx->CR1 |= ADC_CR1_DMAE;
    }
    else
    {
        ADCx->CR1 &= (~ADC_CR1_DMAE);
    }
}

/**
  * @}
  */
