/******************************************************************************
  * @file    PT32x030x_cmp.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2022/12/29
  * @brief   This file provides firmware functions to manage the following
  *          functionalities of the CMP peripheral:
  *           + Initialization and Configuration
  *           + Interrupts and flags management
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "PT32x030x_cmp.h"

/** @defgroup CMP
  * @brief CMP driver modules
  * @{
  */
  
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_CR1_MASK 		((u32)0x007331FC)
#define DeInit_CR_MASK		((u32)0x00000000)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Deinitializes the CMPx peripheral registers to their default reset values.
  * @param  CMPx: where x can be 0 , 1 or 2 to select the CMP peripheral.
  * @retval None
  */
void CMP_DeInit(CMP_TypeDef* CMPx)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	CMPx->CR1 = DeInit_CR_MASK;
	CMPx->CR2 = DeInit_CR_MASK;
}

/**
  * @brief  Initializes the CMPx peripheral according to the specified parameters
  *         in the CMP_InitStruct.
  * @note   This function is used to configure the global features of the CMP.
  * @param  CMPx: where x can be 0 , 1 or 2 to select the CMP peripheral.
  * @param  CMP_InitStruct: pointer to an CMP_InitTypeDef structure that contains
  *         the configuration information for the specified CMP peripheral.
  * @retval None
  */
void CMP_Init(CMP_TypeDef* CMPx, CMP_InitTypeDef* CMP_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_NegativeInput(CMP_InitStruct->CMP_NegativeInput));
	assert_param(IS_CMP_DigitalFilter(CMP_InitStruct->CMP_DigitalFilter));
	assert_param(IS_CMP_PowerMode(CMP_InitStruct->CMP_PowerMode));
	assert_param(IS_CMP_OutputPolarity(CMP_InitStruct->CMP_OutputPolarity));
	assert_param(IS_CMP_Hysteresis(CMP_InitStruct->CMP_Hysteresis));
	assert_param(IS_CMP_PositiveInput(CMP_InitStruct->CMP_PositiveInput));
	assert_param(IS_CMP_InitializationDelayTime(CMP_InitStruct->CMP_InitializationDelayTime));
	/* Write to tmpreg */
	tmpreg = CMPx->CR1;
	tmpreg &= (u32)~Init_CR1_MASK;
	tmpreg  |= ((u32)((CMP_InitStruct->CMP_NegativeInput) | \
										(CMP_InitStruct->CMP_DigitalFilter) | \
										(CMP_InitStruct->CMP_PowerMode)			| \
										(CMP_InitStruct->CMP_OutputPolarity)| \
										(CMP_InitStruct->CMP_Hysteresis)		| \
										(CMP_InitStruct->CMP_PositiveInput)));
	/* Write to CMPx CR1 */
	CMPx->CR1 = tmpreg;
	/* Write to CMPx CR2 */
	CMPx->CR2 = ((u32)(CMP_InitStruct->CMP_InitializationDelayTime));
}

/**
  * @brief  Fills each CMP_InitStruct member with its default value. 
  * @param  CMP_InitStruct: pointer to an CMP_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void CMP_StructInit(CMP_InitTypeDef* CMP_InitStruct)
{
	CMP_InitStruct->CMP_NegativeInput = CMP_NegativeInput_CMPxN;
	CMP_InitStruct->CMP_DigitalFilter = CMP_DigitalFilter_None;
	CMP_InitStruct->CMP_PowerMode = CMP_PowerMode_HighSpeed;
	CMP_InitStruct->CMP_OutputPolarity = CMP_OutputPolarity_Normal;
	CMP_InitStruct->CMP_Hysteresis = CMP_Hysteresis_Disable;
	CMP_InitStruct->CMP_PositiveInput = CMP_PositiveInput_CMPxP0;
	CMP_InitStruct->CMP_InitializationDelayTime = 0x00;
}

/**
  * @brief  Enables or disables the specified CMP peripheral.
  * @param  CMPx: where x can be from 0 , 1 and 2 to select the CMP peripheral.
  * @param  NewState: new state of the CMPx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CMP_Cmd(CMP_TypeDef* CMPx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		CMPx->CR1 |= (u32)CMP_CR1_EN;
	}
	else
	{
		CMPx->CR1 &= (u32)~((u32)CMP_CR1_EN);
	}
}

/**
  * @brief  Enables or disables the specified CMP interrupts.
  * @param  CMPx: where x can be 0 , 1 and 2 to select the CMP peripheral.
  * @param  CMP_IT: specifies the OPA interrupt source to be enabled or disabled.
  *          This parameter can be one of the following values:
  *            @arg CMP_IT_COF: Comparator output falling edge trigger interrupt enable control
  *            @arg CMP_IT_COR: Comparator output rising edge trigger interrupt enable control
  *            @arg CMP_IT_COFW: Comparator output falling edge trigger interrupt enable control(Sleep Mode)
  *            @arg CMP_IT_CORW: Comparator output rising edge trigger interrupt enable control(Sleep Mode)
  * @param  NewState: new state of the specified CMP interrupt.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CMP_ITConfig(CMP_TypeDef* CMPx, u32 CMP_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	assert_param(IS_CMP_IT(CMP_IT));
	if (NewState != DISABLE)
	{
		CMPx->IE |= CMP_IT;
	}
	else
	{
		CMPx->IE &= (~(u32)CMP_IT);
	}
}

/**
  * @brief  Checks whether the specified CMP flag is set or not.
  * @param  CMPx: where x can be 0 , 1 and 2 to select the CMP peripheral.
  * @param  CMP_FLAG: specifies the flag to check.
  *          This parameter can be one of the following values:
  *            @arg CMP_FLAG_COF: Comparator output falling edge trigger interrupt enable control
  *            @arg CMP_FLAG_COR: Comparator output rising edge trigger interrupt enable control
  *            @arg CMP_FLAG_COFW: Comparator output falling edge trigger interrupt enable control(Sleep Mode)
  *            @arg CMP_FLAG_CORW: Comparator output rising edge trigger interrupt enable control(Sleep Mode)
  * @retval The new state of CMP_FLAG (SET or RESET).
  */
FlagStatus CMP_GetFlagStatus(CMP_TypeDef* CMPx, u32 CMP_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_FLAG(CMP_FLAG));
	/* Check the status of the specified ADC flag */
	if ((CMPx->SR & CMP_FLAG) != (u32)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	return  bitstatus;
}

/**
  * @brief  Clears the CMPx's pending flags.
  * @param 	CMPx: where x can be 0 , 1 and 2 to select the CMP peripheral.
  * @param  CMP_FLAG: specifies the flag to clear.
  *          This parameter can be any combination of the following values:
  *            @arg CMP_FLAG_COF: Comparator output falling edge trigger interrupt enable control
  *            @arg CMP_FLAG_COR: Comparator output rising edge trigger interrupt enable control
  *            @arg CMP_FLAG_COFW: Comparator output falling edge trigger interrupt enable control(Sleep Mode)
  *            @arg CMP_FLAG_CORW: Comparator output rising edge trigger interrupt enable control(Sleep Mode)
  * @retval None
  */
void CMP_ClearFlag(CMP_TypeDef* CMPx, u32 CMP_FLAG)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_FLAG(CMP_FLAG));
	CMPx->SR = (u32)CMP_FLAG;
}

/**
  * @brief  Checks whether the specified CMPx's interrupt flag is set or not.
  * @param  CMPx: where x can be 0 , 1 and 2 to select the CMP peripheral.
  * @param  CMP_IT: specifies the IT to check.
  *          This parameter can be one of the following values:
  *            @arg CMP_IT_COF: Comparator output falling edge trigger interrupt enable control
  *            @arg CMP_IT_COR: Comparator output rising edge trigger interrupt enable control
  *            @arg CMP_IT_COFW: Comparator output falling edge trigger interrupt enable control(Sleep Mode)
  *            @arg CMP_IT_CORW: Comparator output rising edge trigger interrupt enable control(Sleep Mode)
  * @retval The new state of CMP_IT (SET or RESET).
  */
ITStatus CMP_GetITStatus(CMP_TypeDef* CMPx, u32 CMP_IT)
{
	ITStatus bitstatus = RESET;
	u32 enablestatus = 0;
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_IT(CMP_IT));
	enablestatus = (u32)(CMPx->IE & CMP_IT);
	if (((u32)(CMPx->SR & CMP_IT) != (u32)RESET) && (enablestatus != (u32)RESET))
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	return  bitstatus;
}

/**
  * @}
  */
