/********************************************************************************
  * @file    PT32x030x_crc.h
  * @author  Application development team
  * @version V1.0.0
  * @date    2022/12/29 
  * @brief   This file contains all the functions prototypes for the CRC firmware 
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

#include "PT32x030x_crc.h"


/** @defgroup CRC
  * @brief CRC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_CR_MASK		((u32)0xFFFFFFC0)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief CRC Initialization
  * @param CRC_InitStruct:CRC_InitTypeDef type structure
  * @retval None
  */
void CRC_Init(CRC_InitTypeDef* CRC_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_CRC_Input(CRC_InitStruct->CRC_Input));
	assert_param(IS_CRC_InputBitSequenceReversal(CRC_InitStruct->CRC_InputBitSequenceReversal));
	assert_param(IS_CRC_InputByteSequenceReversal(CRC_InitStruct->CRC_InputByteSequenceReversal));
	assert_param(IS_CRC_OutputBitSequenceReversal(CRC_InitStruct->CRC_OutputBitSequenceReversal));
    assert_param(IS_CRC_Seed(CRC_InitStruct->CRC_Seed));
	assert_param(IS_CRC_Poly(CRC_InitStruct->CRC_Poly));
	/*  */
	tmpreg = CRC->CR;
	/*  */
	tmpreg &= Init_CR_MASK;
	/*  */
	tmpreg  |= ((u32)(CRC_InitStruct->CRC_Input));
	tmpreg  |= ((u32)(CRC_InitStruct->CRC_InputBitSequenceReversal));
	tmpreg  |= ((u32)(CRC_InitStruct->CRC_InputByteSequenceReversal));
	tmpreg  |= ((u32)(CRC_InitStruct->CRC_OutputBitSequenceReversal));
	/* Write to CRC CR */
	CRC->CR = tmpreg;
	CRC->SEED = CRC_InitStruct->CRC_Seed;
	CRC->POLY = CRC_InitStruct->CRC_Poly;
}

/**
  * @brief CRC Structure Initialization
  * @param CRC_InitStruct:CRC_InitTypeDef type structure
  * @retval None
  */
void CRC_StructInit(CRC_InitTypeDef* CRC_InitStruct)
{	
	/* Initialize the CRC data input width */
	CRC_InitStruct->CRC_Input = CRC_Input_16b;
	/* Initialize the CRC data input bit reverse function */
	CRC_InitStruct->CRC_InputBitSequenceReversal = CRC_InputBitSequenceReversal_Disable;
	/* Initialize the CRC data output bit reverse function */
	CRC_InitStruct->CRC_OutputBitSequenceReversal = CRC_OutputBitSequenceReversal_Disable;
	/* Initialize the CRC data input byte reverse function */
	CRC_InitStruct->CRC_InputByteSequenceReversal = CRC_InputByteSequenceReversal_Enable;
	/* Initialize the CRC POLY */
	CRC_InitStruct->CRC_Poly = CRC_CCITT16;
	/* Initialize the CRC SEED */
	CRC_InitStruct->CRC_Seed = 0;
}

/**
  * @brief CRC command
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void CRC_Cmd(FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the selected CRC peripheral */
		CRC->CR |= CRC_CR_EN;
	}
	else
	{
		/* Disable the selected CRC peripheral */
		CRC->CR &= (u16)~((u16)CRC_CR_EN);
	}
}

/**
  * @brief CRC Reset Dout
  * @retval None
  */
void CRC_ResetDout(void)
{
	CRC->CR |= CRC_CR_CRS;
}

/**
  * @brief CRC Calculate CRC
  * @param CRC_Data:the data will be writed into CRC->DIN
  * @retval CRC->DOUT
  */
u32 CRC_CalculateCRC(u16 CRC_Data)
{
	CRC->DIN = (u16) CRC_Data;
	return (CRC->DOUT);
}

/**
  * @brief CRC Calculate Block CRC
  * @param pBuffer:the data will be writed into CRC->DIN
  * @param BufferLength:the length of pBuffer
  * @retval CRC->DOUT
  */
u32 CRC_CalculateBlockCRC(u16 pBuffer[], u32 BufferLength)
{
	u32 index = 0;
	for(index = 0; index < BufferLength; index++)
	{
		CRC->DIN = pBuffer[index];
	}
	return (CRC->DOUT);
}

/**
  * @brief CRC Get CRC
  * @retval CRC->DOUT
  */
u32 CRC_GetCRC(void)
{
	return (CRC->DOUT);
}

/**
  * @}
  */
