/********************************************************************************
  * @file    PT32x030x_dma.h
  * @author  Application development team
  * @version V1.0.0
  * @date    2022/12/29 
  * @brief   This file contains all the functions prototypes for the DMA firmware 
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/
 
#include "PT32x030x_dma.h"

/** @defgroup DMA
  * @brief DMA driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
u16 DMA_TX_DATA[10] = {1,2,3,4,5,6,7,8,9,10};//!< DMA transfer data
u16 DMA_RX_DATA[10] = {0};//!< DMA receive data

#define Init_CCR_MASK       ((u32)0xFFFFDE61)       //!< CCR MASK 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief Initializes the DMAx peripheral according to
  *         the specified parameters in the DMA_InitStruct.
  * @param DMAx:where x can be 0 or 1 to select the DMA peripheral.
  * @param DMA_InitStruct: pointer to a DMA_InitTypeDef
  *         structure that contains the configuration information for
  *         the specified DMA peripheral.
  * @retval None
  */
void DMA_Init(DMA_TypeDef* DMAx, DMA_InitTypeDef *DMA_InitStruct)
{
    u32 tmpreg = 0;
    /* Check the parameters */
    assert_param(IS_DMA_ALL_PERIPH(DMAx));
    assert_param(IS_DMA_NumberOfData(DMA_InitStruct->DMA_NumberOfData));
    assert_param(IS_DMA_SourceDataSize(DMA_InitStruct->DMA_SourceDataSize));
    assert_param(IS_DMA_DestinationDataSize(DMA_InitStruct->DMA_DestinationDataSize));	
    assert_param(IS_DMA_SourceAddressIncrement(DMA_InitStruct->DMA_SourceAddressIncrement));
    assert_param(IS_DMA_DestinationAddressIncrement(DMA_InitStruct->DMA_DestinationAddressIncrement));
    assert_param(IS_DMA_Direction(DMA_InitStruct->DMA_Direction));
    assert_param(IS_DMA_CircularMode(DMA_InitStruct->DMA_CircularMode));
                  
    /* DMA Channel x Configuration */
    DMAx->CSBAR = DMA_InitStruct->DMA_SourceBaseAddress;
    DMAx->CDBAR = DMA_InitStruct->DMA_DestinationBaseAddress;	
    DMAx->CNDTR = DMA_InitStruct->DMA_NumberOfData;

    /* Get the DMAx CCR value */
    tmpreg = DMAx->CCR;	

    /* Clear Configuration bits */
    tmpreg &= Init_CCR_MASK;	
    tmpreg |= ((DMA_InitStruct->DMA_SourceDataSize) | \
              (DMA_InitStruct->DMA_DestinationDataSize) | \
              (DMA_InitStruct->DMA_SourceAddressIncrement) | \
              (DMA_InitStruct->DMA_DestinationAddressIncrement) | \
              (DMA_InitStruct->DMA_Direction) | \
              (DMA_InitStruct->DMA_CircularMode));                     
    DMAx->CCR = tmpreg;
}

/**
  * @brief Fills each DMA_InitStruct member with its default value.
  * @param DMAx:where x can be 0 or 1 to select the DMA peripheral.
  *         which will be initialized.
  * @retval None
  */
void DMA_StructInit(DMA_InitTypeDef *DMA_InitStruct)
{
	/* USART_InitStruct members default value */
	DMA_InitStruct->DMA_SourceBaseAddress = (uint32_t)DMA_TX_DATA;
	DMA_InitStruct->DMA_DestinationBaseAddress = (uint32_t)DMA_RX_DATA;
	DMA_InitStruct->DMA_NumberOfData = 10;
	DMA_InitStruct->DMA_SourceDataSize = DMA_SourceDataSize_Half;
	DMA_InitStruct->DMA_DestinationDataSize = DMA_DestinationDataSize_Half;
	DMA_InitStruct->DMA_SourceAddressIncrement = DMA_SourceAddressIncrement_Enable;
	DMA_InitStruct->DMA_DestinationAddressIncrement = DMA_DestinationAddressIncrement_Enable;
    DMA_InitStruct->DMA_Direction = DMA_Direction_MemoryToMemory;
    DMA_InitStruct->DMA_CircularMode = DMA_CircularMode_Disable;
}

/**
  * @brief Enable or Disable DMA peripheral.
  * @param DMAx:where x can be 0 or 1 to select the DMA peripheral.
  * @param NewState: This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void DMA_Cmd(DMA_TypeDef* DMAx,FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_DMA_ALL_PERIPH(DMAx));	
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState != DISABLE)
    {
        /* Enable the selected DMA_CH peripheral */
        DMAx->CCR |= DMA_CCR_EN;
    }
    else
    {
        /* Disable the selected DMA_CH peripheral */
        DMAx->CCR &= (u32)~((u32)DMA_CCR_EN);	
    }
}

/**
  * @brief Get DMA's channel current source address.
  * @param DMAx:where x can be 0 or 1 to select the DMA peripheral.
  * @retval DMA channel current source address
  */
u32 DMA_GetSourceAddress(DMA_TypeDef* DMAx)
{
    /* Check the parameters */
    assert_param(IS_DMA_ALL_PERIPH(DMAx));	
    /*Get DMA_CHx Current Source Address */
    return  DMAx->CCSAR;
}

/**
  * @brief Get DMA channel current destination address.
  * @param DMAx:where x can be 0 or 1 to select the DMA peripheral.
  * @retval DMA channel current destination address
  */
u32 DMA_GetDestinationAddress(DMA_TypeDef* DMAx)
{
    /* Check the parameters */
    assert_param(IS_DMA_ALL_PERIPH(DMAx));	
    /*Get DMA_CHx Current Destination Address */
    return  DMAx->CCDAR;
}

/**
  * @brief Get DMA channel number of data.
  * @param DMAx:where x can be 0 or 1 to select the DMA peripheral.
  * @retval DMA channel number of data
  */
u16 DMA_GetNumberOfData(DMA_TypeDef* DMAx)
{
    /* Check the parameters */
    assert_param(IS_DMA_ALL_PERIPH(DMAx));	

    /*Get DMA_CHx Current Destination Address */
    return  DMAx->CCNTR;
}

/**
  * @brief Enables or disables the specified DMA interrupts.
	* @param DMA_IT: Specify the DMA interrupts sources to be enabled or disabled.
  *         This parameter can be one of the following values:
	* 				@arg DMA_IT_TC0E: DMA channel 0 transfer complete interrupt                                      
	* 				@arg DMA_IT_TC1E:	DMA channel 1 transfer complete interrupt 
	* 				@arg DMA_IT_TE0E: DMA channel 0 transfer error interrupt                                     
	*					@arg DMA_IT_TE1E:	DMA channel 1 transfer error interrupt
	*				 	@arg DMA_IT_CEE: DMA channel configuration error interrupt
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void DMA_ITConfig(u32 DMA_IT, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_DMA_IT(DMA_IT));

    if (NewState != DISABLE)
    {
        /* Enable the selected QSPI interrupt */
        DMACM->IER |= DMA_IT;
    }
    else
    {
        /* Disable the selected QSPI interrupt */
        DMACM->IER &= (u32)~DMA_IT;
    }
}

/**
  * @brief Clear DMA interrupt flag
	* @param DMA_FLAG: Specify the DMA interrupts flag to be clear.
  *         This parameter can be one of the following values:
	* 				@arg DMA_IT_TC0E: DMA channel 0 transfer complete interrupt                                      
	* 				@arg DMA_IT_TC1E:	DMA channel 1 transfer complete interrupt 
	* 				@arg DMA_IT_TE0E: DMA channel 0 transfer error interrupt                                     
	*					@arg DMA_IT_TE1E:	DMA channel 1 transfer error interrupt
	*				 	@arg DMA_IT_CEE: DMA channel configuration error interrupt
  * @retval None
  */
void DMA_ClearITFlag(u32 DMA_FLAG)
{
    /* Check the parameters */
    assert_param(IS_DMA_IT(DMA_FLAG));

    DMACM->ICR = DMA_FLAG;
}

/**
  * @brief DMA Get Flag Status
  * @param DMA_FLAG: Specify the flag to be checked.
  *         This parameter can be one of the following values:
	* 				@arg DMA_FLAG_TC0F: DMA channel 0 transfer complete interrupt flag                                       
	* 				@arg DMA_FLAG_TC1F: DMA channel 1 transfer complete interrupt flag
	* 				@arg DMA_FLAG_TE0F: DMA channel 0 transfer error interrupt flag
	* 				@arg DMA_FLAG_TE1F: DMA channel 1 transfer error interrupt flag                                       
	* 				@arg DMA_FLAG_BSY0: DMA channel 0 busy flag
	* 				@arg DMA_FLAG_BSY1: DMA channel 1 busy flag
	* 				@arg DMA_FLAG_CEF: DMA channel configuration error interrupt flag
  * @return FlagStatus of DMA_FLAG (SET or RESET).
  */
FlagStatus DMA_GetFlagStatus(u32 DMA_FLAG)
{
    /* Check the parameters */
    assert_param(IS_DMA_FLAG(DMA_FLAG));	

    if ((DMACM->SR & DMA_FLAG) != 0)
    {
        return SET;
    }
    else
    {
        return RESET;
    }
}

/**
  * @brief Config DMA Peripheral
  * @param DMAx:where x can be 0 or 1 to select the DMA peripheral.
  * @param Peripheral: DAM CH Relevance Peripheral.
  *         This parameter can be one of the following values:
	*   			@arg DMA_ChannelxPERIPH_UART0RX: DMA connect with UART0 receive   
	*   			@arg DMA_ChannelxPERIPH_UART0TX: DMA connect with UART0 transmit   
	*   			@arg DMA_ChannelxPERIPH_UART1RX: DMA connect with UART1 receive       
	*   			@arg DMA_ChannelxPERIPH_UART1TX: DMA connect with UART1 transmit      
	*   			@arg DMA_ChannelxPERIPH_SPI0RX: DMA connect with SPI0 receive   	
	*   			@arg DMA_ChannelxPERIPH_SPI0TX: DMA connect with SPI0 transmit  
	*   			@arg DMA_ChannelxPERIPH_SPI1RX: DMA connect with SPI1 receive
	*   			@arg DMA_ChannelxPERIPH_SPI1TX: DMA connect with SPI1 transmit	
	*   			@arg DMA_ChannelxPERIPH_TIMER1: DMA connect with timer1   	
	*   			@arg DMA_ChannelxPERIPH_TIMER5: DMA connect with timer5  	
	*   			@arg DMA_ChannelxPERIPH_TIMER6: DMA connect with timer6  	
	*  			  @arg DMA_ChannelxPERIPH_TIMER7: DMA connect with timer7
	*   			@arg DMA_ChannelxPERIPH_ADC: DMA connect with ADC conversion
  * @retval None
  */
void DMA_PeripheralConfig(DMA_TypeDef* DMAx,u32 Peripheral)
{
    /* Check the parameters */
    assert_param(IS_DMA_ALL_PERIPH(DMAx));
    assert_param(IS_DMA_ChannelxPERIPH(Peripheral));	
    if(DMAx == DMA0)
    {
        DMACM->CHCR &= (u32)(~DMA_CHAPCR_CHSEL);	
        DMACM->CHCR |= Peripheral;
    }
    else
    {
        DMACM->CHCR &= (u32)(~(DMA_CHAPCR_CHSEL<<8));	
        DMACM->CHCR |= (Peripheral<<8);
    }
}

/**
  * @}
  */
