/******************************************************************************
  * @file    PT32x030x_ifmc.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2022/12/29
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the IFMC peripheral:
  *           + Initialization and Configuration
  *           + Flash Erase and Read\Write Function
  *           + Interrupts and flags management
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x030x_ifmc.h"

/** @defgroup IFMC
  * @brief IFMC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define IFMC_CR1_MASK           ((u32)0XFFFFFEF8)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Sets the code latency value.
  * @param  IFMC_Latency: specifies the IFMC Latency value.
  *          This parameter can be one of the following values:
  *             @arg IFMC_Latency_0: IFMC Zero Latency cycle
  *             @arg IFMC_Latency_1: IFMC One Latency cycle
	*             @arg IFMC_Latency_2: IFMC Two Latency cycle
  * @retval None
  */
void IFMC_SetLatency(u32 IFMC_Latency)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_IFMC_Latency(IFMC_Latency));
	/* Read the ACR register */
	tmpreg = IFMC->CR;
	/* Sets the Latency value */
	tmpreg &= ~(IFMC_CR_WAIT);
	tmpreg |= IFMC_Latency;
	/* Write the ACR register */
	IFMC->CR = tmpreg;
}

/**
  * @brief  Sets the IFMC Prescaler.
  * @param  Prescaler: specifies the IFMC PSC value.
  *		    	The clock frequency of IFMC = the clock frequency of SYS / IFMC Prescaler
	*         @note The clock frequency of IFMC sholudn't over 1MHz
  * @retval None
  */
void IFMC_SetPrescaler(u8 Prescaler)
{
	/* Check the parameters */
	assert_param(IS_IFMC_Prescaler(Prescaler));
	/* Write the PSC register */
	IFMC->PSC =Prescaler;
}

/**
  * @brief  Returns the IFMC Status.
  * @param  None
  * @retval IFMC Status: The returned value can be:
	*         @arg IFMC_Busy: Operation busy status
	*         @arg IFMC_KeyError: Key error status
	*         @arg IFMC_AddrError: Address error status
	*         @arg IFMC_Complete:  Complete status
  */
IFMC_Status IFMC_GetIFMCStatus(void)
{
	IFMC_Status IFMCstatus = IFMC_Complete;
	if((IFMC->SR & IFMC_SR_BSY) == IFMC_SR_BSY)
	{
		IFMCstatus = IFMC_Busy;
	}
	else
	{
		if((IFMC->SR & IFMC_SR_KERR) == IFMC_SR_KERR)
		{
			IFMCstatus = IFMC_KeyError;
		}
		else
		{
			if((IFMC->SR & IFMC_SR_AERR) == IFMC_SR_AERR)
			{
				IFMCstatus = IFMC_AddrError;
			}
			else
			{
				IFMCstatus = IFMC_Complete;
			}
		}
	}
	/* Return the IFMC Status */
	return IFMCstatus;
}

/**
  * @brief  Waits for a IFMC operation to complete or a TIMEOUT to occur.
  * @param  Timeout: IFMC programming Timeout
  * @retval IFMC_Status: The returned value can be: IFMC_Busy,
  *         IFMC_KeyError, IFMC_AddrError ,IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_WaitForLastOperation(u32 Timeout)
{
	IFMC_Status status = IFMC_Complete;
	/* Check for the IFMC Status */
	status = IFMC_GetIFMCStatus();
	/* Wait for a IFMC operation to complete or a TIMEOUT to occur */
	while((IFMC_GetIFMCStatus() == IFMC_Busy) && (Timeout != 0x00))
	{
		status = IFMC_GetIFMCStatus();
		Timeout--;
	}
	if(Timeout == 0x00 )
	{
		status = IFMC_TimeOut;
	}
	/* Return the operation status */
	return status;
}

/**
  * @brief  Erases a specified page in program memory.
  * @param  Page_Address: The page address in program memory to be erased.
  * @note   A Page is erased in the Program memory only if the address to load
  *         is the start address of a page (multiple of 1024 bytes).
  * @retval IFMC_Status: The returned value can be: IFMC_Busy,
  *         IFMC_KeyError, IFMC_AddrError ,IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_ErasePage(u32 Page_Address)
{
	u32 tmpreg = 0;
	IFMC_Status status = IFMC_Complete;
	/* Check the parameters */
	assert_param(IS_IFMC_PageAddress(Page_Address));
	/* Wait for last operation to be completed */
	status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);	//ȴһ
	tmpreg = IFMC->CR;
	tmpreg &= IFMC_CR1_MASK;
	if(status == IFMC_Complete)
	{
		IFMC->AR  = Page_Address;
		if(((Page_Address) < MAINCODE_END_ADDR))	//ַСBoot ŵַΧ(ڵĵַ)
		{
			/* The main code area is erased by page */
			tmpreg |= IFMC_MainCodeKey | IFMC_WriteOperation | IFMC_PageErase | IFMC_OperationStarts;
			IFMC->CR =tmpreg;
			status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
		}
		else											//ַBoot ŵַΧ(ûϢڵĵַ)
		{
			status = IFMC_AddrError;
		}
	}
	/* Return the Erase Status */
	return status;
}

/**
  * @brief  Erases all IFMC pages.
  * @param  None
  * @retval IFMC_Status: The returned value can be: IFMC_Busy,
  *         IFMC_KeyError, IFMC_AddrError ,IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_EraseAllPages(void)
{
	u32 tmpreg = 0;	
	IFMC_Status status = IFMC_Complete;
	tmpreg = IFMC->CR;	
	tmpreg &= IFMC_CR1_MASK;
	/* Wait for last operation to be completed */
	status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
	if(status == IFMC_Complete)
	{
		/* if the previous operation is completed, proceed to erase all pages */
	  tmpreg |= IFMC_MainCodeKey | IFMC_WriteOperation | IFMC_MainCodeErase | IFMC_OperationStarts;
		IFMC->CR =tmpreg;		
		/* Wait for last operation to be completed */
		status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
	}
	/* Return the Erase Status */
	return status;
}

/**
  * @brief  Programs a word at a specified address.
  * @param  Address: specifies the address to be programmed.
  * @param  Data: specifies the data to be programmed.
  * @retval IFMC_Status: The returned value can be: IFMC_Busy,
  *         IFMC_KeyError, IFMC_AddrError ,IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_ProgramWord(u32 Address, u32 Data)
{
	IFMC_Status status = IFMC_Complete;
	__IO u32 tmp = 0;
	u32 tmpreg = 0;
	tmpreg = IFMC->CR;	
	tmpreg &= IFMC_CR1_MASK;
	/* Check the parameters */
	//assert_param(IS_IFMC_PROGRAM_ADDRESS(Address));
	/* Wait for last operation to be completed */
	status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
	if(status == IFMC_Complete)
	{
		/* if the previous operation is completed, proceed to erase all pages */
		IFMC->AR  = Address;
		IFMC->DR  = Data;
		if(((Address) < MAINCODE_END_ADDR) && ((Address & 0x03) == 0))
		{
			/* Data is programming into the main code area */
			tmpreg |=IFMC_MainCodeKey | IFMC_WriteOperation | IFMC_Programming | IFMC_OperationStarts;
			IFMC->CR = tmpreg;
			status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
		}
		else
		{
			status = IFMC_AddrError;
		}
	}
	/* Return the Program Status */
	return status;
}

/**
  * @brief  Read a word at a specified address.
  * @param  Address: specifies the address to be read.
  * @retval The word at a specified address.
  */
u32 IFMC_ReadWord(u32 Address)
{
	return *(__IO u32*) Address;
}

/**
  * @brief  Read a half word at a specified address.
  * @param  Address: specifies the address to be read.
  * @retval The half word at a specified address.
  */
u16 IFMC_ReadHalfWord(u32 Address)
{
	return *(__IO u16*) Address;
}

/**
  * @brief Read a Byte at a specified address.
  * @param  Address: specifies the address to be read.
  * @retval The byte at a specified address.
  */
u8 IFMC_ReadByte(u32 Address)
{
	return *(__IO u8*) Address;
}

/**
  * @brief  Enables or disables the specified IFMC interrupts.
  * @param  IFMC_IT: specifies the IFMC interrupt sources to be enabled or
  *         disabled.
  *          This parameter can be any combination of the following values:
	*          @arg IFMC_IT_WCIE: Write operation complete interrupt
	*          @arg IFMC_IT_PECIE: Page erase operation complete interrupt	  
	*          @arg IFMC_IT_CECIE: Chip erase operation complete interrupt	 
	*          @arg IFMC_IT_CERIE: Operation command error interrupt
	*          @arg IFMC_IT_KERIE: Key error interrupt
	*          @arg IFMC_IT_AERIE: Address error interrupt
	* @param  NewState: new state of the I2Cx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void IFMC_ITConfig(u32 IFMC_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_IFMC_IT(IFMC_IT));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if(NewState != DISABLE)
	{
		/* Enable the interrupt sources */
		IFMC->IE |= IFMC_IT;
	}
	else
	{
		/* Disable the interrupt sources */
		IFMC->IE &= ~(u32)IFMC_IT;
	}
}

/**
  * @brief  Checks whether the specified IFMC flag is set or not.
  * @param  IFMC_FLAG: specifies the IFMC flag to check.
  *          This parameter can be one of the following values:
	*					 @arg IFMC_FLAG_WC: Write operation complete flag
	*					 @arg IFMC_FLAG_PEC: Page erase operation complete flag
	*					 @arg IFMC_FLAG_CEC: Chip erase operation complete flag
	*					 @arg IFMC_FLAG_BSY: Operation busy flag
	*					 @arg IFMC_FLAG_CERR: Operation command error flag
	*					 @arg IFMC_FLAG_KERR: Key error flag
	*					 @arg IFMC_FLAG_AERR: Address error flag
  * @retval The new state of IFMC_FLAG (SET or RESET).
  */
FlagStatus IFMC_GetFlagStatus(u32 IFMC_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_IFMC_GETFLAG(IFMC_FLAG));
	if((IFMC->SR & IFMC_FLAG) != (u32)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	/* Return the new state of IFMC_FLAG (SET or RESET) */
	return bitstatus;
}

/**
  * @brief  Clears the IFMC's pending flags.
  * @param  IFMC_FLAG: specifies the IFMC flags to clear.
  *          This parameter can be any combination of the following values:
	*					 @arg IFMC_FLAG_WC: Write operation complete flag
	*					 @arg IFMC_FLAG_PEC: Page erase operation complete flag
	*					 @arg IFMC_FLAG_CEC: Chip erase operation complete flag
	*					 @arg IFMC_FLAG_BSY: Operation busy flag
	*					 @arg IFMC_FLAG_CERR: Operation command error flag
	*					 @arg IFMC_FLAG_KERR: Key error flag
	*					 @arg IFMC_FLAG_AERR: Address error flag
	*					 @arg IFMC_FLAG_ALL: All flags above
  * @retval None
  */
void IFMC_ClearFlag(u32 IFMC_FLAG)
{
	/* Check the parameters */
	assert_param(IS_IFMC_ClearFLAG(IFMC_FLAG));
	/* Clear the flags */
	IFMC->SR = IFMC_FLAG;
}

/**
  * @brief  Returns the ReadProtect Status.
  * @param  None
  * @retval The new state of ReadProtect Status (SET or RESET).
  */
ProtectStatus IFMC_GetReadProtectStatus(void)
{
	u32 tmp;
	tmp = (IFMC->RPSR & IFMC_RPSR_PSR);
	if (tmp != RESET)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/**
  * @}
  */
