  /******************************************************************************
  * @file    PT32x030x_uart.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2022/12/29
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the Universal synchronous asynchronous receiver
  *          transmitter (USART):
  *           + Initialization and Configuration
  *           + STOP Mode
  *           + BaudRate
  *           + Data transfers
  *           + Multi-Processor Communication
  *           + Half-duplex mode
  *           + Smartcard mode
  *           + Interrupts and flags management
  *            
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "PT32x030x_uart.h"

/** @defgroup UART
  * @brief UART driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_CR_MASK       ((u32)0xFFECEE00) 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

 
/**
  * @brief  Initializes the UARTx peripheral according to the specified
  *         parameters in the USART_InitStruct .
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  UART_InitStruct: pointer to a UART_InitTypeDef structure that contains
  *         the configuration information for the specified UART peripheral.
  * @retval None
  */
void UART_Init(UART_TypeDef* UARTx, UART_InitTypeDef* UART_InitStruct)
{
	u32  tmpreg = 0, pclk = 0;
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_BaudRate(UART_InitStruct->UART_BaudRate));
	assert_param(IS_UART_WordLengthAndParity(UART_InitStruct->UART_WordLengthAndParity));
	assert_param(IS_UART_StopBitLength(UART_InitStruct->UART_StopBitLength));
	assert_param(IS_UART_ParityMode(UART_InitStruct->UART_ParityMode));
	assert_param(IS_UART_Receiver(UART_InitStruct->UART_Receiver));
  assert_param(IS_UART_LoopbackMode(UART_InitStruct->UART_LoopbackMode));
  /* Clear Register */
	UARTx->RXFR = 0x00;
	UARTx->TXFR = 0x00;
	/* Wirite to CR */
  tmpreg = UARTx->CR;
	tmpreg &= Init_CR_MASK;
	tmpreg |= 	(UART_InitStruct ->UART_WordLengthAndParity | \
				UART_InitStruct ->UART_StopBitLength | \
				UART_InitStruct ->UART_ParityMode | \
				UART_InitStruct->UART_Receiver | \
				UART_InitStruct->UART_LoopbackMode);
	UARTx->CR = tmpreg;
	/* Get the BaudRate  */
	pclk = RCC_GetClockFreq(RCC_PCLK);
	UARTx->BRR = ( (pclk / 16) / UART_InitStruct->UART_BaudRate);
}

/**
  * @brief  Fills each UART_InitStruct member with its default value.
  * @param  UART_InitStruct: pointer to a UART_InitTypeDef structure
  *         which will be initialized.
  * @retval None
  */
void UART_StructInit(UART_InitTypeDef* UART_InitStruct)
{
	/* USART_InitStruct members default value */
	UART_InitStruct->UART_BaudRate = 19200;
	UART_InitStruct->UART_WordLengthAndParity = UART_WordLengthAndParity_8DP;
	UART_InitStruct->UART_StopBitLength = UART_StopBitLength_1;
	UART_InitStruct->UART_ParityMode = UART_ParityMode_Odd;
	UART_InitStruct->UART_LoopbackMode = UART_LoopbackMode_Disable;
    UART_InitStruct->UART_Receiver = UART_Receiver_Enable;
}

/**
  * @brief  Enables or disables the specified UART peripheral.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  NewState: new state of the USARTx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_Cmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the selected USART by setting the UE bit in the CR1 register */
		UARTx->CR |= UART_CR_EN;
	}
	else
	{
		/* Disable the selected USART by clearing the UE bit in the CR1 register */
		UARTx->CR &= (u32)~((u32)UART_CR_EN);
	}
}

/**
  * @brief  Sets the receiver Time Out value.
  * @param  UARTx: where x can be 0 or 1 to select the UART peripheral.
  * @param  UART_ReceiverTimeOut: specifies the Receiver Time Out value(0x00~0xFF).
  * @retval None
  */
void UART_SetTimeOut(UART_TypeDef* UARTx, u32 TimeOut)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_TimeOut(TimeOut));
	/* Clear the receiver Time Out value by clearing the RTO[23:0] bits in the RTOR
	   register  */
	UARTx->TO &= (u32)~((u32)UART_TO_TO);
	/* Set the receiver Time Out value by setting the RTO[23:0] bits in the RTOR
	   register  */
	UARTx->TO |= TimeOut;
}

/**
  * @brief  Transmits single data through the UARTx peripheral.
  * @param  UARTx: where x can be from 0 or 1 to select the UART peripheral.
  * @param  Data: the data to transmit.
  * @retval None
  */
void UART_SendData(UART_TypeDef* UARTx, u16 Data)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_Data(Data));
	/* While the TxFIFO contain 8 characters. */
	while((UARTx->SR & UART_SR_TXF));
	/* Transmit Data */
	UARTx->DR = (Data & (u16)0x01FF);
}

/**
  * @brief  Returns the most recent received data by the UARTx peripheral.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @retval The received data.
  */
u16 UART_ReceiveData(UART_TypeDef* UARTx)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	/* Receive Data */
	return (u16)(UARTx->DR & (u16)0x01FF);
}

/**
  * @brief  This function reset the Rx and the Tx FIFOs
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  FIFO: Transmit FIFO or receive FIFO for the UART .
  *          This parameter can be:
  *            @arg Rx_FIFO: Receive FIFO .
  *            @arg Tx_FIFO: Transmit FIFO .
  * @retval None.
  */
void UART_FifoReset(UART_TypeDef * UARTx,  u8 UART_FIFO)
{
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_UART_FIFO(UART_FIFO));
	switch (UART_FIFO)
	{
		case FIFO_RX :
			UARTx->RXFR = UART_RXFR_RXFR;
			break;
		case FIFO_TX :
			UARTx->TXFR = UART_TXFR_TXFR;
			break;
	}
}

/**
  * @brief  Enables or disables the UART's IrDA interface.
  * @param  UARTx: where x can be 0 to select the UART peripheral.
  * @param  NewState: new state of the IrDA mode.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_IrDACmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_USART_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the IrDA mode by setting the EN bit in the IRR register */
		UARTx->IMCR |= UART_IMCR_IRE;
	}
	else
	{
		/* Disable the IrDA mode by clearing the EN bit in the IRR register */
		UARTx->IMCR &= (u32)~((u32)UART_IMCR_IRE);
	}
}

/**
  * @brief  Configures the IrDA's Pin polarity
  * @param  UARTx: where x can be 0 to select the UART peripheral.
  * @param  NewState: new defined levels for the USART data.
  *          This parameter can be one of the following values:
  *           @arg ENABLE: pin(s) signal values are inverted (Vdd =0, Gnd =1).
	*           @arg DIABLE: pin(s) signal works using the standard logic levels (Vdd =1, Gnd =0).
  * @note   This function has to be called before calling UART_Cmd() function.
  * @retval None
  */
void UART_IrDAPolarityConfig(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_USART_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the binary data inversion feature by setting the DATAINV bit in
		   the CR2 register */
		UARTx->IMCR |= UART_IMCR_IRPN;
	}
	else
	{
		/* Disable the binary data inversion feature by clearing the DATAINV bit in
		   the CR2 register */
		UARTx->IMCR &= (u32)~((u32)UART_IMCR_IRPN);
	}
}

/**
  * @brief  Set the Infrared Baud Rate for UART peripheral.
  * @param  UARTx: where x can be 0 to select the UART peripheral.
  * @param  IrDABaudRate: set infrared baud rate for UART peripheral.
  *          This parameter can be one of the following values:
  *           @arg UART_InfraredBaudRate_600bps: set 600 bits per second as the infrared baud rate   
	*           @arg UART_InfraredBaudRate_1200bps: set 1200 bits per second as the infrared baud rate 
	*           @arg UART_InfraredBaudRate_2400bps: set 1400 bits per second as the infrared baud rate
  * @note   This function has to be called before calling UART_Cmd() function.
  * @retval None
  */
void UART_IrDABaudRateConfig(UART_TypeDef* UARTx, u8 IrDABaudRate)
{
	/* Check the parameters */
	assert_param(IS_USART_PERIPH(UARTx));
  assert_param(IS_UART_InfraredBaudRate(IrDABaudRate));
	/* Write to IMCR */
  UARTx->IMCR &= ~UART_IMCR_IRBD;
  UARTx->IMCR |= ((u32)IrDABaudRate);

}

/**
  * @brief  Sets the infrared duty cycle for UART peripheral.
  * @param  UARTx: where x can be 0 to select the UART peripheral.
  * @param  IrDADutyCycle: specifies the infrared duty cycle.
  * @note   This function has to be called before calling UART_Cmd() function.
  * @retval None
  */
void UART_IrDADutyCycleConfig(UART_TypeDef* UARTx, u8 IrDADutyCycle)
{
	/* Check the parameters */
	assert_param(IS_USART_PERIPH(UARTx));
	assert_param(IS_UART_DutyCycle(IrDADutyCycle));
	/* Clear the IrDA's modulation PWM duty cycle */
	UARTx->IMDCR &= ~UART_IMDCR_DUTY;
	/* Set the IrDA's modulation PWM duty cycle*/
	UARTx->IMDCR |= (u32)((u32)IrDADutyCycle);
}

/**
  * @brief  Enables or disables the UART's Half Duplex communication.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  NewState: new state of the UART Communication.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_HalfDuplexCmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the Half-Duplex mode by setting the OWE bit in the CR register */
		UARTx->CR |= UART_CR_SLME;
	}
	else
	{
		/* Disable the Half-Duplex mode by clearing the OWE bit in the CR register */
		UARTx->CR  &= (u32)~((u32)UART_CR_SLME);
	}
}

/**
  * @brief  Enables or disables the UART's transmitter or receiver.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  UART_Direction: specifies the UART direction.
  *          This parameter can be any combination of the following values:
  *            @arg UART_SingleLineDirection_Tx: UART Single Line Direction Transmitter
  *            @arg UART_SingleLineDirection_Rx: UART Single Line Direction Receiver
  * @param  NewState: new state of the USART transfer direction.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */

void UART_HalfDuplexDirectionConfig(UART_TypeDef* UARTx, u32 Direction)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_SingleLineDirection(Direction));
	if (Direction != UART_SingleLineDirection_Rx)
	{
		/* Set NSS pin internally by software */
		UARTx->CR |= UART_SingleLineDirection_Tx;
	}
	else
	{
		/* Reset NSS pin internally by software */
		UARTx->CR &= UART_SingleLineDirection_Rx;
	}
}

/**
  * @brief  Enables or disables the specified UART interrupts.
  * @param  UARTx: where x can be 0 or 1 to select the UART peripheral.
  * @param  UART_IT: specifies the UART interrupt sources to be enabled or disabled.
  *          This parameter can be one of the following values:
  *            @arg UART_IT_RXNE: specifies the interrupt source for Receiver FIFO buffer non-empty interrupt.
  *            @arg UART_IT_RXHF: specifies the interrupt source for Receiver FIFO buffer half-full interrupt.
  *            @arg UART_IT_RXF: specifies the interrupt source for Receiver FIFO buffer full interrupt.
  *            @arg UART_IT_RXCOT: specifies the interrupt source for Receive Receiver FIFO buffer cleared overtime interrupt.
  *            @arg UART_IT_IDLE: specifies the interrupt source for Receiver FIFO buffer idel interrupt.
  *            @arg UART_IT_PE: specifies the interrupt source for Parity error interrupt.
  *            @arg UART_IT_FE: specifies the interrupt source for Frame error interrupt.
  *            @arg UART_IT_OVR: specifies the interrupt source for Receiver FIFO buffer overflow interrupt.
  *            @arg UART_IT_TXE: specifies the interrupt source for Transmitter FIFO buffer empty interrupt.
  *            @arg UART_IT_TXHE: specifies the interrupt source for Transmitter FIFO buffer half-empty interrupt.
  *            @arg UART_IT_TXF: specifies the interrupt source for Transmitter FIFO buffer full interrupt.
  *            @arg UART_IT_TXO: specifies the interrupt source for Transfer completed interrupt
  * @param  NewState: new state of the specified UARTx interrupts.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_ITConfig(UART_TypeDef* UARTx, u32 UART_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_IT(UART_IT));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		UARTx->IE |= UART_IT;
	}
	else
	{
		UARTx->IE &= ~UART_IT;
	}
}

/**
  * @brief  Checks whether the specified UART flag is set or not.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  UART_FLAG: specifies the flag to check.
  *          This parameter can be one of the following values:
  *            @arg UART_FLAG_RXNE: Receive FIFO not empty flag.
  *            @arg UART_FLAG_RXHF: Receive FIFO half full flag.
  *            @arg UART_FLAG_RXF: Receive FIFO full flag.
  *            @arg UART_FLAG_RXCOT: Receive FIFO clear overtime flag.
  *            @arg UART_FLAG_IDLE: Idle overtime flag.
  *            @arg UART_FLAG_PE: Parity error flag.
  *            @arg UART_FLAG_FE: Frame error flag.
  *            @arg UART_FLAG_OVR: Receive FIFO overflow flag.
  *            @arg UART_FLAG_TXE: Transmit FIFO empty flag.
  *            @arg UART_FLAG_TXHE: Transmit FIFO half empty flag.
  *            @arg UART_FLAG_TXF: Transmit FIFO full flag.
  *            @arg UART_FLAG_TXO: Transmit end flag.
  * @retval The new state of UART_FLAG (SET or RESET).
  */
FlagStatus UART_GetFlagStatus(UART_TypeDef* UARTx, u32 UART_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_FLAG(UART_FLAG));
	//assert_param(IS_UART_SR_SEL(UART_FLAG));
	if ((UARTx->SR & UART_FLAG) != (u16)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	return bitstatus;
}

/**
  * @brief  Clears the UARTx's pending flags.
  * @param  UARTx: where x can be from 0 to 1 to select the USART peripheral.
  * @param  UART_CLEAR_FLAG: specifies the flag to clear.
  *          This parameter can be any combination of the following values:
  *            @arg UART_FLAG_PE:  Parity error.
  *            @arg UART_FLAG_FE:  Frame error.
  *            @arg UART_FLAG_OVR: Receiver FIFO buffer overflow.
  * @retval None
  */
void UART_ClearFlag(UART_TypeDef* UARTx, u32 UART_CLEAR_FLAG)
{
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_FLAG(UART_CLEAR_FLAG));
	UARTx->SR = UART_CLEAR_FLAG;
}

/**
  * @brief  Checks whether the specified UART intterrupt status is set or not.
  * @param  UARTx: where x can be from 0 to 1 to select the USART peripheral.
  * @param  UART_IT: Specify the intterrupt status to be checked.
  *         This parameter can be one of the following values:
	*            @arg UART_IT_RXNE: specifies the interrupt source for Receiver FIFO buffer non-empty interrupt.
  *            @arg UART_IT_RXHF: specifies the interrupt source for Receiver FIFO buffer half-full interrupt.
  *            @arg UART_IT_RXF: specifies the interrupt source for Receiver FIFO buffer full interrupt.
  *            @arg UART_IT_RXCOT: specifies the interrupt source for Receive Receiver FIFO buffer cleared overtime interrupt.
  *            @arg UART_IT_IDLE: specifies the interrupt source for Receiver FIFO buffer idel interrupt.
  *            @arg UART_IT_PE: specifies the interrupt source for Parity error interrupt.
  *            @arg UART_IT_FE: specifies the interrupt source for Frame error interrupt.
  *            @arg UART_IT_OVR: specifies the interrupt source for Receiver FIFO buffer overflow interrupt.
  *            @arg UART_IT_TXE: specifies the interrupt source for Transmitter FIFO buffer empty interrupt.
  *            @arg UART_IT_TXHE: specifies the interrupt source for Transmitter FIFO buffer half-empty interrupt.
  *            @arg UART_IT_TXF: specifies the interrupt source for Transmitter FIFO buffer full interrupt.
  *            @arg UART_IT_TXO: specifies the interrupt source for Transfer completed interrupt
  * @return ITStatus of UART_IT (SET or RESET).
  */
ITStatus UART_GetITStatus(UART_TypeDef* UARTx, u32 UART_IT)
{
	ITStatus bitstatus = RESET;
	u32 enablestatus = 0;
	/* Check the parameters */
	assert_param(IS_UART_ALL_PERIPH(UARTx));
	assert_param(IS_UART_IT(UART_IT));
	/**/
	enablestatus = (u32)(UARTx->IE & UART_IT);
	/*  */
	if (((u32)(UARTx->SR & UART_IT) != (u32)RESET) && (enablestatus != (u32)RESET))
	{
		/* */
		bitstatus = SET;
	}
	else
	{
		/* */
		bitstatus = RESET;
	}
	/**/
	return  bitstatus;
}

/**
  * @}
  */

