  /******************************************************************************
  * @file    PT32x030x_wdg.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2022/12/29
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the Independent watchdog (WDG) peripheral:
  *           + Prescaler and Counter configuration
  *           + WDG activation
  *           + Flag management
  *            
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "PT32x030x_wdg.h"

/** @defgroup WDG
  * @brief WDG driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Enables or disables write access all registers.
  * @param  WDGx:select the WDG peripheral.
  * @param  SEL: new state of write access to all registers.
  *          This parameter can be one of the following values:
  *            @arg WDG_LOCK_UnLock: Enable write access to all registers
  *            @arg WDG_LOCK_Lock: Disable write access to all registers
  * @retval None
  */
void WDG_LockConfig(WDG_TypeDef* WDGx, u32 SEL)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_WDG_LockKey(SEL));
	WDGx->LOCK = SEL;
}

/**
  * @brief  Sets WDG Reload value.
  * @param  WDGx:select the WDG peripheral.
  * @param  Reload: specifies the WDG Reload value.
  *          This parameter must be a number between 0x00 and 0xFFFFFFFF.
  * @retval None
  */
void WDG_SetReload(WDG_TypeDef* WDGx, u32 Reload)
{
	/* Check the parameters */
	assert_param(IS_WDG_ReloadValue(Reload));
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	WDGx->RLR = Reload;
}

/**
  * @brief  Reloads WDG counter 
  * @param  WDGx:select the WDG peripheral.
  * @retval None
  */
void WDG_ReloadCounter(WDG_TypeDef* WDGx)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	WDGx->KR = 1;
}

/**
  * @brief  Enable or Disable WDGx
  * @param  WDGx:select the WDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void WDG_Cmd(WDG_TypeDef* WDGx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the WDG */
		WDGx->CR |= WDG_CR_EN;
	}
	else
	{
		/* Disable the WDG */
		WDGx->CR &= ~WDG_CR_EN;
	}
}

/**
  * @brief  Enables or disables the specified WDG Reset.
  * @param  WDGx:select the WDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void WDG_ResetConfig(WDG_TypeDef* WDGx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the WDG RESET */
		WDGx->CR |= WDG_CR_RSTE;
	}
	else
	{
		/* Disable the WDG RESET*/
		WDGx->CR &= ~WDG_CR_RSTE;
	}
}

/**
  * @brief  Enables or disables the specified WDG DebugPending
  * @param  WDGx:select the WDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void WDG_DebugPendingConfig(WDG_TypeDef* WDGx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if (NewState != DISABLE)
	{
		/* Enable the WDG RESET */
		WDGx->CR |= WDG_CR_DBGE;
	}
	else
	{
		/* Disable the WDG RESET*/
		WDGx->CR &= ~WDG_CR_DBGE;
	}
}

/**
  * @brief  Checks whether the specified WDG flag is set or not.
  * @param  WDGx:select the WDG peripheral.
  * @param  WDG_FLAG: specifies the flag to check.
  *          This parameter can be one of the following values:
  *            @arg WDG_SR_HDF
  * @retval The new state of WDG_FLAG (SET or RESET).
  */
FlagStatus WDG_GetFlagStatus(WDG_TypeDef* WDGx, WDG_FLAG Flag)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_WDG_FLAG(Flag));
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
    if(Flag == WDG_FLAG_HungryDog)
    {
			if ((WDGx->SR & WDG_LOCK_LOCK) != (u32)RESET)
			{
					bitstatus = SET;
			}
			else
			{
					bitstatus = RESET;
			}
    }
    else if(Flag == WDG_FLAG_LockStatus)
    {
			if ((WDGx->LOCK & WDG_LOCK_LOCK) != (u32)RESET)
			{
					bitstatus = SET;
			}
			else
			{
					bitstatus = RESET;
			}
    }
	
	/* Return the flag status */
	return bitstatus;
}

/**
  * @}
  */
